/*
 * menu_whatson.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "menu_whatson.h"
#include "epgsearchext.h"
#include "menu_event.h"
#include "menu_myedittimer.h"
#include "menu_searchresults.h"
#include "menu_search.h" 
#include "menu_commands.h"
#include "epgsearchcfg.h"
#include "tools.h"
#include "switchtimer.h"
#include <math.h>
#include <vdr/menu.h>

#define HOURS(x) ((x)/100)
#define MINUTES(x) ((x)%100)

extern int exitToMainMenu;

// --- cMenuMyScheduleItem ------------------------------------------------------
cMenuMyScheduleItem::cMenuMyScheduleItem(const cEvent *Event, cChannel *Channel, int Progress, showMode Mode)
{
  event = Event;
  channel = Channel;
  progress = Progress;
  mode = Mode;
  timerMatch = tmNone;
  inSwitchList = false;
  Update(true);
}

bool cMenuMyScheduleItem::Update(bool Force)
{
  bool result = false;

  int OldTimerMatch = timerMatch;
  bool OldInSwitchList = inSwitchList;
  bool hasMatch = false; 
  if (event) hasMatch = Timers.GetMatch(event, &timerMatch);
  if (event) inSwitchList = (SwitchTimers.InSwitchList(event)!=NULL);

  if (Force || timerMatch != OldTimerMatch || inSwitchList != OldInSwitchList) 
  {
      char szChannelNr[6] = "";
      if (channel && EPGSearchConfig.showChannelNr)
	  sprintf(szChannelNr, "%d\t", channel->Number());
      
      char szChannelpart[20] = "";
      if (channel)
	  sprintf(szChannelpart, "%s%.*s\t", szChannelNr, 10, CHANNELNAME(channel));
      
      char szProgressPart[12] = "";
      if (progress > 0 && channel)
      {
	  strcpy(szProgressPart, "\t");
	  if (event)
	  {
	      char szProgress[9] = "";
	      int frac = 0;
	      if (mode == showNow)
		  frac = (int)roundf( (float)(time(NULL) - event->StartTime()) / (float)(event->Duration()) * 8.0 );
	      if (mode == showNext)
		  frac = (int)roundf( (float)(1 - min(float(30*60), float(event->StartTime() - time(NULL))) / (30*60)) * 8.0 );
	      frac = min(8,max(0, frac));
	      
	      for(int i = 0; i < frac; i++)
		  szProgress[i] = (progress == 1 ? '|' : 127);
	      szProgress[frac]=0;
	      sprintf(szProgressPart, "%c%-8s%c\t", progress==1?'[':128, szProgress, progress==1?']':129);
	  }
      }
      
      char t = event && hasMatch ? (timerMatch == tmFull) ? 'T' : 't' : ' ';
      char v = event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
      char r = event && event->IsRunning() ? '*' : ' ';
      
      if (event && inSwitchList)
      {
	  cSwitchTimer* s = SwitchTimers.InSwitchList(event);
	  t = (s && s->announceOnly)?'s':'S';
      }
      char szEventDescr[100] = "";
      snprintf(szEventDescr, 100, "%s%s%s", 
	      event?event->Title():tr(">>> no info! <<<"), 
	      SHORTTEXT(event));

      char *buffer = NULL;
      if (channel) // menu "What's on"
	  asprintf(&buffer, "%s%s\t%s %c%c%c \t%s", 
		   szChannelpart,
		   event?GETTIMESTRING(event):"", 
		   szProgressPart,
		   t, v, r, 
		   szEventDescr);
      else // menu "Schedule"
	  asprintf(&buffer, "%.*s\t%s\t%c%c%c\t%s", 
		   6, GETDATESTRING(event), GETTIMESTRING(event), 
		   t, v, r, 
		   szEventDescr);

      SetText(buffer, false);
      return true;
  }
  return result;
}
  
// --- cMenuWhatsOnSearch ----------------------------------------------------------

int cMenuWhatsOnSearch::currentChannel = 0;
showMode cMenuWhatsOnSearch::currentShowMode = showNow;
cChannel *cMenuWhatsOnSearch::scheduleChannel = NULL;
extern const char *ShowModes[];

cMenuWhatsOnSearch::cMenuWhatsOnSearch(const cSchedules *Schedules, int CurrentChannelNr)
    :cOsdMenu(EPGSearchConfig.ShowModes[currentShowMode].GetDescription(), GetTab(1), GetTab(2), GetTab(3), GetTab(4), GetTab(5))
{
    shiftTime = 0;
    schedules = Schedules;

    LoadSchedules();

    currentChannel = CurrentChannelNr;
    
    char* szTitle = NULL;
    asprintf(&szTitle, "%s - %s", tr("Overview"), EPGSearchConfig.ShowModes[currentShowMode].GetDescription());
    SetTitle(szTitle);
    free(szTitle);
    SetHelpKeys();
}

cMenuWhatsOnSearch::~cMenuWhatsOnSearch()
{
}

int cMenuWhatsOnSearch::GetTab(int Tab)
{
    if (Tab==1)
	return EPGSearchConfig.showChannelNr?CHNUMWIDTH:6+CHNUMWIDTH;
    if (Tab==2)
	return EPGSearchConfig.showChannelNr?10:6;
    if (Tab==3)
	if (currentShowMode==showNow || currentShowMode==showNext)
	    return EPGSearchConfig.showChannelNr?7:EPGSearchConfig.ProgressWidth;
	else
	    return EPGSearchConfig.showChannelNr?5:4;
    if (Tab==4)
	return EPGSearchConfig.showChannelNr?EPGSearchConfig.ProgressWidth:4;
    if (Tab==5)
	return 4;
    return 0;
}

void cMenuWhatsOnSearch::LoadSchedules()
{
    Clear();

    time_t SeekTime;
    char* szTitle = NULL;
    if (shiftTime != 0)
    {
	if (currentShowMode == showNow || currentShowMode == showNext)
	    SeekTime = time(NULL);
	else
	    SeekTime = GetTimeT(EPGSearchConfig.ShowModes[currentShowMode].GetTime());
	SeekTime += shiftTime*60;

	struct tm tm_r;
	time_t now = time(NULL);
	tm tm_seek = *localtime_r(&SeekTime, &tm_r);
	tm tm_now = *localtime_r(&now, &tm_r);
	if (tm_seek.tm_mday != tm_now.tm_mday)
	    asprintf(&szTitle, "%s - %s", tr("Overview"), DAYDATETIME(SeekTime));
	else
	    asprintf(&szTitle, "%s - %02d:%02d", tr("Overview"), tm_seek.tm_hour, tm_seek.tm_min);
    }
    else
	asprintf(&szTitle, "%s - %s", tr("Overview"), EPGSearchConfig.ShowModes[currentShowMode].GetDescription());
    
    SetTitle(szTitle);
    free(szTitle);
	
    for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
	if (!Channel->GroupSep()) {
	    const cSchedule *Schedule = schedules->GetSchedule(Channel->GetChannelID());
	    const cEvent *Event = NULL;
	    if (Schedule) 
	    {		
		if (shiftTime != 0)
		    Event = Schedule->GetEventAround(SeekTime);
		else
		{
		    switch(currentShowMode)
		    {  
			default:
			case showNow: 
			    Event = Schedule->GetPresentEvent();
			    break;
			case showNext:
			    Event = Schedule->GetFollowingEvent();
			    break;
			case showUserMode1:
			case showUserMode2:
			case showUserMode3:
			case showUserMode4:
			    time_t SeekTime = GetTimeT(EPGSearchConfig.ShowModes[currentShowMode].GetTime());
			    Event = Schedule->GetEventAround(SeekTime);
			    break;
		    }
		}
	     }
	     if (!EPGSearchConfig.showEmptyChannels && !Event)
		 continue;
	     int iProgress = 0;
	     if (currentShowMode==showNow)
		 iProgress = EPGSearchConfig.showProgress;
	     if (currentShowMode==showNext)
		 iProgress = EPGSearchConfig.showProgressNext;
	     Add(new cMenuMyScheduleItem(Event, Channel, iProgress, currentShowMode), Channel->Number() == currentChannel);
	}
	else
	{
	    if (EPGSearchConfig.showChannelGroups)
	    {
		char* szGroup = NULL;
		asprintf(&szGroup, "----------------------------------------\t %s ----------------------------------------------------------------------------------------------", Channel->Name());
		cOsdItem* pGroupItem = new cOsdItem(szGroup);
		free(szGroup);
		pGroupItem->SetSelectable(false);
		Add(pGroupItem);
	    }
	}
    }
}

time_t cMenuWhatsOnSearch::GetTimeT(int iTime)
{
    struct tm tm_r;
    time_t t = time(NULL);
    tm* tmnow = localtime_r(&t, &tm_r);
    tmnow->tm_hour = HOURS(iTime);
    tmnow->tm_min = MINUTES(iTime);
    return mktime(tmnow);
}

showMode cMenuWhatsOnSearch::GetNextMode()
{
    showMode nextShowMode = currentShowMode;
    time_t now = time(NULL);
    do
    {
	nextShowMode = (showMode)((nextShowMode+1) % showModeMax);
	if (nextShowMode >= showUserMode1)
	{
	    if (!EPGSearchConfig.ShowModes[nextShowMode].GetUsage())
		continue;

	    time_t TestTime = GetTimeT(EPGSearchConfig.ShowModes[nextShowMode].GetTime());    
	    if (TestTime < now)
		continue;
	    break;
	}
	else
	    break;
    }
    while(true);
    return nextShowMode;
}
cChannel *cMenuWhatsOnSearch::ScheduleChannel(void)
{
  cChannel *ch = scheduleChannel;
  scheduleChannel = NULL;
  return ch;
}


eOSState cMenuWhatsOnSearch::Switch(void)
{
  cMenuMyScheduleItem *item = (cMenuMyScheduleItem *)Get(Current());
  if (item && item->channel) {
     if (cDevice::PrimaryDevice()->SwitchChannel(item->channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuWhatsOnSearch::Record(void)
{
  cMenuMyScheduleItem *item = (cMenuMyScheduleItem *)Get(Current());
  if (item) {
      cTimer *timer = NULL;
      if (item->event)
      {
          timer = new cTimer(item->event);
          AddSubtitleToTimerFile(item->event, timer);
      }
      else
      {
          time_t now = time(NULL); // thanks to nordlicht@vdrportal for the following code, that
          char *buffer=NULL;       // allows setting the channel for a timer without event
          int starthh,startmm,stophh,stopmm;
          //for getting the day of month
          struct tm *tm_t=localtime(&now);
          //we have to get away the ':' from the time string
          sscanf(TIMESTRING(now),"%2d:%2d",&starthh,&startmm);
          sscanf(TIMESTRING(now+Setup.InstantRecordTime*60),"%2d:%2d",&stophh,&stopmm);
          //active:channel:day_of_month:starttime:endtime:priority:durability:title:
          asprintf(&buffer,"1:%d:%d:%d%d:%d%d:%d:%d:%s:",item->channel->Number(),tm_t->tm_mday,starthh,startmm,stophh,stopmm,Setup.DefaultPriority,Setup.DefaultLifetime,item->channel->Name());
          timer=new cTimer(false,false);
          if(timer->Parse(buffer))
              free(buffer);
      }
      cTimer *t = Timers.GetTimer(timer);
      if (t) {
          delete timer;
          timer = t;
      }
      timer->SetFlags(tfActive);
#if VDRVERSNUM && VDRVERSNUM >= 10331
      if (EPGSearchConfig.useVDRTimerEditMenu)
          return AddSubMenu(new cMenuEditTimer(timer, !t));
      else
#endif
          return AddSubMenu(new cMenuMyEditTimer(timer, !t, item->event, item->channel));
  }
  return osContinue;
}

bool cMenuWhatsOnSearch::Update(void)
{
  bool result = false;
  for (cOsdItem *item = First(); item; item = Next(item)) {
      if (item->Selectable() && ((cMenuMyScheduleItem *)item)->Update())
         result = true;
      }
  return result;
}

eOSState cMenuWhatsOnSearch::Commands(eKeys Key)
{
    if (HasSubMenu() || Count() == 0)
	return osContinue;

    cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
    if (mi) 
    {
	if (!mi->event) 
	{
	    if (Key == k3)
		return Switch();
	    else if (Key == k2)
		return Record();
	    else
		return osContinue;
	}
        cMenuSearchCommands *menu;
        eOSState state = AddSubMenu(menu = new cMenuSearchCommands(tr("EPG Commands"), mi->event, true));
        if (Key != kNone)
	    state = menu->ProcessKey(Key);
        return state;
    }
    return osContinue;
}

eOSState cMenuWhatsOnSearch::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

void cMenuWhatsOnSearch::SetHelpKeys()
{
    cMenuMyScheduleItem *item = (cMenuMyScheduleItem *)Get(Current());
    bool hasTimer = item && item->Selectable() && item->timerMatch == tmFull;

    showMode nextShowMode = GetNextMode();    
    char* szButtonGreen = EPGSearchConfig.ShowModes[nextShowMode].GetDescription();
  if (toggleKeys==0)
      SetHelp((EPGSearchConfig.redkeymode==0?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), szButtonGreen, tr("Button$Schedule"), EPGSearchConfig.bluekeymode==0?tr("Button$Switch"):tr("Button$Search"));
  else
      SetHelp((EPGSearchConfig.redkeymode==1?(hasTimer?tr("Button$Timer"):tr("Button$Record")):tr("Button$Commands")), (EPGSearchConfig.toggleGreenYellow==0?szButtonGreen:"<<"), (EPGSearchConfig.toggleGreenYellow==0?tr("Button$Schedule"):">>"), EPGSearchConfig.bluekeymode==1?tr("Button$Switch"):tr("Button$Search"));
}

eOSState cMenuWhatsOnSearch::Shift(int iMinutes)
{
    shiftTime += iMinutes;
    cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
    int TempChannel = currentChannel;
    if (mi) 
    {
	currentChannel = mi->channel->Number();
	scheduleChannel = Channels.GetByNumber(currentChannel);
    }
    LoadSchedules();
    Display();
    currentChannel = TempChannel;
    SetHelpKeys();
    return osContinue;
}

eOSState cMenuWhatsOnSearch::ShowSummary()
{
    if (Count())
    {
	const cEvent *ei = ((cMenuMyScheduleItem *)Get(Current()))->event;
	if (ei) 
	{
	    cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
	    if (channel)
		return AddSubMenu(new cMenuEventSearch(ei, channel->Number() != cDevice::CurrentChannel(), GetNextChannelWithEPG(-1), GetNextChannelWithEPG(1)));
	}
    }
    return osContinue;
}

const cChannel* cMenuWhatsOnSearch::GetNextChannelWithEPG(int direction)
{
    int currentOld = Current();
    int current = currentOld;
    do
    {	
	if (direction>0) current++; else current--;
	if (current<0 || current>=Count()) return NULL;
	cMenuMyScheduleItem* item = (cMenuMyScheduleItem *)Get(current);
	if (!item) return NULL;

	if (currentOld == current) // no move
	    return NULL;
	if (item->Selectable() && item->event)
	    return item->channel;
    }
    while(true);
}

void cMenuWhatsOnSearch::MoveToChannelWithEPG(int direction)
{
    
    const cChannel* ch = NULL;
    do
    {
	int currentOld = Current();
	if (direction>0)
	    CursorDown();
	else
	    CursorUp();
	if (currentOld == Current()) // no move
	    return;
	const cEvent *event = ((cMenuMyScheduleItem *)Get(Current()))->event;	
	if (event)
	    return;
	else
	    ch = ((cMenuMyScheduleItem *)Get(Current()))->channel;
    }
    while(ch);
}


eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
{
    exitToMainMenu = 0;
    if (!HasSubMenu() && Key == kBack)
    {
	exitToMainMenu = 1;
	return osBack;
    }

    bool HadSubMenu = HasSubMenu();
    eOSState state = cOsdMenu::ProcessKey(Key);

    if (state == osUnknown) {
	switch (Key) {
	    case kFastRew:
		if(HasSubMenu()) 	     
		{
		    MoveToChannelWithEPG(-1);
		    return ShowSummary();
		}
		else
		    return Shift(-EPGSearchConfig.timeShiftValue);
	    case kFastFwd:
		if(HasSubMenu()) 	     
		{
		    MoveToChannelWithEPG(1);
		    return ShowSummary();
		}
		else
		    return Shift(EPGSearchConfig.timeShiftValue);
	    case kRecord:
	    case kRed:
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
      		if (Count())
		{
			if (EPGSearchConfig.redkeymode==toggleKeys)
				state = Record();
			else
			{
				cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
	                        if (mi) {
        	                  if (mi->event) 
				     return AddSubMenu(new cMenuSearchCommands(tr("EPG Commands"),mi->event));
				  else
				      return osContinue;
				}
			}
		}
		break;

	 case k0:
	     if(!HasSubMenu()) 
	     {
		 toggleKeys = 1 - toggleKeys;
		 SetHelpKeys();
	     }
	     state = osContinue;
	     break;
	 case k1...k9: return Commands(Key);
	 case kYellow: 
	     if(HasSubMenu()) 	     
	     {
		 MoveToChannelWithEPG(1);
		 return ShowSummary();
	     }
	     else if (toggleKeys == 0 || (toggleKeys == 1 && EPGSearchConfig.toggleGreenYellow == 0))
	     {
		 cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
		 if (mi && mi->channel)
		 {
		     const cSchedule *Schedule = schedules->GetSchedule(mi->channel->GetChannelID());
		     if (Schedule)
		     {
			 time_t now = time(NULL);
			 bool hasFutureEvents = false;
			 for (const cEvent *e = Schedule->Events()->First(); e; e = Schedule->Events()->Next(e))
			     if (e->StartTime() > now)
			     {
				 esyslog("Start: %s", GETDATESTRING(e));
				 hasFutureEvents = true;
				 break;
			     }
			 if (!hasFutureEvents)
			     return osContinue;
		     }
		     else
			 return osContinue;
		 }
		 state = osBack;
		 // continue with kGreen
	     }
	     else
		 return Shift(EPGSearchConfig.timeShiftValue);
	 case kGreen:  
	 {
	     if(HasSubMenu()) 	     
	     {
		 MoveToChannelWithEPG(-1);
		 return ShowSummary();
	     }
	     else if (toggleKeys == 0 || (toggleKeys == 1 && EPGSearchConfig.toggleGreenYellow == 0))
	     {
		 if (Key == kYellow)
		     currentShowMode = showNow;
		 else
		     currentShowMode = GetNextMode();
		 cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
		 if (mi) 
		 {
		     currentChannel = mi->channel->Number();
		     scheduleChannel = Channels.GetByNumber(currentChannel);
		 }
	     }
	     else
		 return Shift(-EPGSearchConfig.timeShiftValue);
	 }
	 break;
	 case kBlue:   
	     if (HasSubMenu())
		 return Switch();
	     if (EPGSearchConfig.bluekeymode==toggleKeys)
		 return Switch();
	     else
		 return ExtendedSearch();
	   break;
	 case kOk:
	 {
	     cMenuMyScheduleItem *mi = (cMenuMyScheduleItem *)Get(Current());
	     if (!mi->event) // no EPG, so simply switch to channel
		 return Switch();
	     else
		 return ShowSummary();
	 }
	 break;
       default:      break;
     }
    }
    if (!HasSubMenu()) 
    {
	if ((HadSubMenu || gl_TimerProgged) && Update())
	{
	    if (gl_TimerProgged) // when using epgsearch's timer edit menu, update is delayed because of SVDRP
	    {
		gl_TimerProgged = 0;
		SetHelpKeys();
	    }
	    Display();
	}
	if (Key != kNone)
	    SetHelpKeys();
    }
    
    return state;
}


